import BaseFoundation, { DefaultAdapter } from '../base/foundation';
import isElement from '../utils/isElement';
import { slice, find, findIndex } from 'lodash';
import { append, prepend } from '../utils/dom';

export interface Item {
    [x: string]: any;
    transform?: (value: any, text: string) => string;
    value: any;
    text?: string;
    disabled?: boolean;
}

export interface ScrollItemAdapter<P = Record<string, any>, S = Record<string, any>, I = Item>
    extends DefaultAdapter<P, S> {
    setPrependCount: (prependCount: number) => void;
    setAppendCount: (appendCount: number) => void;
    setSelectedNode: (el: HTMLElement) => void;
    isDisabledIndex: (i: number) => boolean;
    notifySelectItem: (data: I) => void;
    notifySelectEndItem: (data: I) => void;
    scrollToCenter: (selectedNode: Element, scrollWrapper?: Element, duration?: number) => void;
    setRangeEnd: (rangeHourEnd: any) => void;
    setHoverHourTimer: (hoverHour: any) => void;
    setRangeStart: (rangeHourStart: any) => void;
    setHourList: (list: any) => void;
    setMinutesList: (list: any) => void;
    setNeedReselect: (needReselect: any) => void;
    notifyEndHour: (index: any) => void;
    notifyStartHour: (index: any) => void;
    setHoverMinuteTimer: (hoverHour: any) => void;
    setRangeMinuteStart: (index: any) => void;
    setRangeMinuteEnd: (index: any) => void;
    setMinutesNeedReselect: (index: any) => void;
    notifyMinutesStart: (index: any) => void;
    notifyMinutesEnd: (index: any) => void
}

export default class ItemTimerFoundation<P = Record<string, any>, S = Record<string, any>, I = Item> extends BaseFoundation<ScrollItemAdapter<P, S, I>,
P,
S
> {
    _cachedSelectedNode: HTMLElement = null;

    constructor(adapter) {
        super({ ...adapter });
    }

    setHoverHourHandle(index) {
        this._adapter.setHoverHourTimer(index);
    }

    formatOption(option: number) {
        let value = `${option}`;
        if (option < 10) {
            value = `0${option}`;
        }

        return {
            value,
        };
    }

    initData() {
        let hourOptionsAdj = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23];
        this._adapter.setHourList(hourOptionsAdj.map(option => this.formatOption(option)));
    }
    initMinutesData() {
        let hourOptionsAdj = [
            0,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            10,
            11,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19,
            20,
            21,
            22,
            23,
            24,
            25,
            26,
            27,
            28,
            29,
            30,
            31,
            32,
            33,
            34,
            35,
            36,
            37,
            38,
            39,
            40,
            41,
            42,
            43,
            44,
            45,
            46,
            47,
            48,
            49,
            50,
            51,
            52,
            53,
            54,
            55,
            56,
            57,
            58,
            59,
        ];
        this._adapter.setMinutesList(hourOptionsAdj.map(option => this.formatOption(option)));
    }

    setRange(index) {
        const { hoverHour, rangeHourStart, rangeHourEnd, needReselect } = this.getStates();
        if (needReselect) {
            this._adapter.setRangeStart(index);
            this._adapter.setRangeEnd(index);
            this._adapter.setNeedReselect(false);
            this._adapter.notifyStartHour(index);
            // this._adapter.notifyEndHour(index);
        } else {
            this._adapter.setNeedReselect(true);
            this._adapter.setRangeEnd(index);
            this._adapter.notifyEndHour(index);
        }
        console.log(rangeHourStart, rangeHourEnd, 'RAngeSTart');
    }

    setHoverMinuteHandle(index) {
        this._adapter.setHoverMinuteTimer(index);
    }
    setMinutesRange(index) {
        const { hoverHour, rangeHourStart, rangeHourEnd, needReselect } = this.getStates();
        console.log(hoverHour, needReselect, 'NeeDReslected');
        if (needReselect) {
            this._adapter.setRangeMinuteStart(index);
            this._adapter.setRangeMinuteEnd(index);
            this._adapter.setMinutesNeedReselect(false);
            this._adapter.notifyMinutesStart(index);
        } else {
            this._adapter.setMinutesNeedReselect(true);
            this._adapter.setRangeMinuteEnd(index);
            this._adapter.notifyMinutesEnd(index);
        }
    }
}
